/*
 * par: simple paragraph formater
 *
 * (C) 1998 by Kamil Toman (e-mail: toman@horac.ta.jcu.cz)
 *
 * 24th April 1999
 * version: 1.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as by published
 * by the Free Software Foundation; either version 2; or (at your option)
 * any later version
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Publis License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "interfac.h"

#include "getopt.h"

unsigned char *progname, *inputrc = NULL;

static struct option long_options[] =
{
    {"left", REQUIREDARG, NULL, 'l'},
    {"right", REQUIREDARG, NULL, 'r'},
    {"parindent", REQUIREDARG, NULL, 'p'},
    {"hangindent", REQUIREDARG, NULL, 'h'},
    {"words", REQUIREDARG, NULL, 'w'},
    {"version", NOARG, NULL, 'v'},
    {"help", NOARG, NULL, 'h'},
    {0, 0, 0, 0}
};

void parse_cmdline_parameters(int argc, char **argv, int param[NUMOFPARAMS])
{
    int c, optind = 0, err = 0;

    ALLOC(progname, strlen(argv[0]) + 1);
    strcpy(progname, argv[0]);

    LEFT = IMPLICIT_LEFT;
    RIGHT = IMPLICIT_RIGHT;
    PARINDENT = IMPLICIT_PARINDENT;
    HANGINDENT = IMPLICIT_HANGINDENT;

    while (1) {
	c = getopt_long(argc, argv, "-l:r:p:g:w:hv", long_options, &optind);
	if (c == EOF)
	    break;

	switch (c) {
	case 1:
	    fprintf(stderr, WINVALIDPAR, progname, optarg);
	    err = 1;
	    break;
	case 'l':
	    ASSIGN_PAR(LEFT, optarg);
	    break;
	case 'r':
	    ASSIGN_PAR(RIGHT, optarg);
	    break;
	case 'p':
	    ASSIGN_PAR(PARINDENT, optarg);
	    break;
	case 'g':
	    ASSIGN_PAR(HANGINDENT, optarg);
	    break;
	case 'w':
	    ALLOC(inputrc, strlen(optarg) + 1);
	    strcpy(inputrc, optarg);
	    break;
	case 'h':
	    err_exit(EXIT_SUCCESS, USAGE);
	    break;
	case 'v':
	    fprintf(stderr, VERSION);
	    break;
	case '?':
	    fprintf(stderr, WUNKNOWNOPT, progname, argv[optind]);
	    err = 1;
	    break;
	case ':':
	    fprintf(stderr, WMISSINGPAR, progname);
	    err = 1;
	    break;
	default:
	    err_exit(EXIT_FAILURE, ERRUNKNOWN);
	}
    }

    if (err)
	err_exit(EXIT_FAILURE, ERRUSAGE);

    if (LEFT < 0 || HANGINDENT <= 0 || (LEFT + PARINDENT) < 0
	|| (LEFT + PARINDENT > RIGHT || LEFT >= RIGHT))
	err_exit(EXIT_FAILURE, ERRRANGE);

    if (inputrc == NULL)
	inputrc = getenv("PARRC");

    LEFT += PARINDENT;
    RIGHT -= LEFT;
}

void err_exit(int code, const char *messg)
{
    fprintf(stderr, "%s: %s", progname, messg);
    exit(code);
}
